using System.Collections.Generic;
using System.Linq;
namespace Oxide.Plugins
{
    [Info("Realistic Weather", "OuTSMoKE", "1.1.4", ResourceId = 2519)]
    [Description("Calls weather events with console commands.")]
    class RealisticWeather : RustPlugin
    {
        public Timer StormTimer;
        public int time = 0;
		public List<string> cmds = new List<string>
        {
            "rw rain",
            "rw lightrain",
            "rw longrain",
			"rw storm",
			"rw longstorm",
			"rw lightfog",
			"rw fog",
			"rw heavyfog",
			"rw overcast"
        };

        void Unload()
        {
            if (StormTimer != null)
                StormTimer.Destroy();
            time = 0;
        }

        [ConsoleCommand("rw")]
        void cmdSpecial(ConsoleSystem.Arg arg)
        {
            if (!arg.IsServerside) return;
            if (arg.Args == null || arg.Args.Count() == 0) return;
            var stormType = arg.Args[0];
            if (stormType.Equals("random"))
            {
                stormType = Storms.Keys.ToList().GetRandom();
                arg.ReplyWith($"Running random storm: {stormType}");
            }
            if (!Storms.ContainsKey(stormType)) return;
            NewStormTimer(stormType);
        }

        void NewStormTimer(string TypeOfStorm)
        {
            if (StormTimer != null)
                StormTimer.Destroy();
            time = 0;
            StormTimer = timer.Every(1, () => ProcessStorm(Storms[TypeOfStorm]));
        }

        void ProcessStorm(Dictionary<int, List<string>> StormData)
        {
            time++;
            if (StormData.ContainsKey(time))
                foreach (var entry in StormData[time])
                {
                    if (entry == "STOP") { StormTimer.Destroy(); time = 0; return; }
                    ConsoleSystem.Run(ConsoleSystem.Option.Server, entry);
                }
        }

        public Dictionary<string, Dictionary<int, List<string>>> Storms = new Dictionary<string, Dictionary<int, List<string>>>
                {
                    {"storm", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.02" } },
						{10, new List<string> {"weather.clouds 0.04" } },
						{20, new List<string> {"weather.clouds 0.06" } },
						{30, new List<string> {"weather.clouds 0.08" } },
						{40, new List<string> {"weather.clouds 0.1" } },
						{50, new List<string> {"weather.clouds 0.12" } },
						{60, new List<string> {"weather.clouds 0.14" } },
						{70, new List<string> {"weather.clouds 0.16" } },
						{80, new List<string> {"weather.clouds 0.18" } },
						{90, new List<string> {"weather.clouds 0.2" } },
						{100, new List<string> {"weather.clouds 0.22" } },
						{110, new List<string> {"weather.clouds 0.24" } },
						{120, new List<string> {"weather.clouds 0.26" } },
						{130, new List<string> {"weather.clouds 0.28" } },
						{140, new List<string> {"weather.clouds 0.3" } },
						{150, new List<string> {"weather.clouds 0.32" } },
						{160, new List<string> {"weather.clouds 0.34" } },
						{170, new List<string> {"weather.clouds 0.36" } },
						{180, new List<string> {"weather.clouds 0.38" } },
						{190, new List<string> {"weather.clouds 0.4" } },
						{200, new List<string> {"weather.clouds 0.42" } },
						{210, new List<string> {"weather.clouds 0.44" } },
						{220, new List<string> {"weather.clouds 0.46" } },
						{230, new List<string> {"weather.clouds 0.48" } },
						{240, new List<string> {"weather.clouds 0.5" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.clouds 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.clouds 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.clouds 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.clouds 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.clouds 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.clouds 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.clouds 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.clouds 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.clouds 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.clouds 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.clouds 0.74" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.clouds 0.76" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{520, new List<string> {"weather.clouds 0.78" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{540, new List<string> {"weather.clouds 0.8" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{560, new List<string> {"weather.clouds 0.82" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{580, new List<string> {"weather.clouds 0.84" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{600, new List<string> {"weather.clouds 0.86" } },
						{605, new List<string> {"weather.wind 0.2" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{620, new List<string> {"weather.clouds 0.88" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{640, new List<string> {"weather.clouds 0.9" } },
						{650, new List<string> {"weather.fog 0.42" } },
						{660, new List<string> {"weather.clouds 0.92" } },
						{670, new List<string> {"weather.fog 0.44" } },
						{680, new List<string> {"weather.clouds 0.94" } },
						{690, new List<string> {"weather.fog 0.46" } },
						{700, new List<string> {"weather.clouds 0.96" } },
						{710, new List<string> {"weather.fog 0.48" } },
						{720, new List<string> {"weather.clouds 0.98" } },
						{725, new List<string> {"weather.wind 0.4" } },
						{730, new List<string> {"weather.fog 0.5" } },
						{740, new List<string> {"weather.clouds 1" } },
						{750, new List<string> {"weather.fog 0.52" } },
						{760, new List<string> {"weather.fog 0.54" } },
						{780, new List<string> {"weather.fog 0.56" } },
						{790, new List<string> {"weather.fog 0.58" } },
						{800, new List<string> {"weather.fog 0.6" } },
						{810, new List<string> {"weather.fog 0.62" } },
						{820, new List<string> {"weather.fog 0.64" } },
						{830, new List<string> {"weather.fog 0.66" } },
						{840, new List<string> {"weather.fog 0.68" } },
						{850, new List<string> {"weather.fog 0.7", "weather.wind 0.6" } },
						{860, new List<string> {"weather.fog 0.72" } },
						{870, new List<string> {"weather.fog 0.74" } },
						{880, new List<string> {"weather.fog 0.76" } },
						{890, new List<string> {"weather.fog 0.78" } },
						{900, new List<string> {"weather.fog 0.8" } },
						{905, new List<string> {"weather.wind 0.8" } },
						{910, new List<string> {"weather.fog 0.82" } },
						{920, new List<string> {"weather.fog 0.84" } },
						{930, new List<string> {"weather.fog 0.86" } },
						{940, new List<string> {"weather.fog 0.88" } },
						{950, new List<string> {"weather.fog 0.9" } },
						{960, new List<string> {"weather.fog 0.92" } },
						{965, new List<string> {"weather.wind 1" } },
						{970, new List<string> {"weather.fog 0.94" } },
						{980, new List<string> {"weather.fog 0.96" } },
						{990, new List<string> {"weather.fog 0.98" } },
						{1000, new List<string> {"weather.fog 1" } },
						{1300, new List<string> {"weather.fog 0.98" } },
						{1310, new List<string> {"weather.fog 0.96" } },
						{1320, new List<string> {"weather.fog 0.94" } },
						{1330, new List<string> {"weather.fog 0.92" } },
						{1340, new List<string> {"weather.fog 0.9" } },
						{1350, new List<string> {"weather.fog 0.88" } },
						{1360, new List<string> {"weather.fog 0.86" } },
						{1365, new List<string> {"weather.wind 0.8" } },
						{1370, new List<string> {"weather.fog 0.84" } },
						{1380, new List<string> {"weather.fog 0.82" } },
						{1390, new List<string> {"weather.fog 0.8" } },
						{1395, new List<string> {"weather.wind 0.6" } },
						{1400, new List<string> {"weather.fog 0.78" } },
						{1410, new List<string> {"weather.fog 0.76" } },
						{1420, new List<string> {"weather.fog 0.74" } },
						{1430, new List<string> {"weather.fog 0.72" } },
						{1440, new List<string> {"weather.fog 0.7" } },
						{1450, new List<string> {"weather.fog 0.68" } },
						{1460, new List<string> {"weather.fog 0.66", "weather.wind 0.4" } },
						{1470, new List<string> {"weather.fog 0.64" } },
						{1480, new List<string> {"weather.fog 0.62" } },
						{1490, new List<string> {"weather.fog 0.6" } },
						{1500, new List<string> {"weather.fog 0.58" } },
						{1510, new List<string> {"weather.fog 0.56" } },
						{1520, new List<string> {"weather.fog 0.54" } },
						{1530, new List<string> {"weather.fog 0.52" } },
						{1540, new List<string> {"weather.fog 0.5", "weather.wind 0.2" } },
						{1550, new List<string> {"weather.clouds 0.98" } },
						{1560, new List<string> {"weather.fog 0.48" } },
						{1570, new List<string> {"weather.clouds 0.96" } },
						{1580, new List<string> {"weather.fog 0.46" } },
						{1590, new List<string> {"weather.clouds 0.94" } },
						{1600, new List<string> {"weather.fog 0.44" } },
						{1610, new List<string> {"weather.clouds 0.92" } },
						{1620, new List<string> {"weather.fog 0.42" } },
						{1630, new List<string> {"weather.clouds 0.9" } },
						{1635, new List<string> {"weather.wind 0" } },
						{1640, new List<string> {"weather.fog 0.4" } },
						{1650, new List<string> {"weather.clouds 0.88" } },
						{1660, new List<string> {"weather.fog 0.38" } },
						{1670, new List<string> {"weather.clouds 0.86" } },
						{1680, new List<string> {"weather.fog 0.36" } },
						{1690, new List<string> {"weather.clouds 0.84" } },
						{1700, new List<string> {"weather.fog 0.34" } },
						{1710, new List<string> {"weather.clouds 0.82" } },
						{1720, new List<string> {"weather.fog 0.32" } },
						{1730, new List<string> {"weather.clouds 0.8" } },
						{1740, new List<string> {"weather.fog 0.3" } },
						{1750, new List<string> {"weather.clouds 0.78" } },
						{1760, new List<string> {"weather.fog 0.28" } },
						{1770, new List<string> {"weather.clouds 0.76" } },
						{1780, new List<string> {"weather.fog 0.26" } },
						{1790, new List<string> {"weather.clouds 0.74" } },
						{1800, new List<string> {"weather.fog 0.24" } },
						{1810, new List<string> {"weather.clouds 0.72" } },
						{1820, new List<string> {"weather.fog 0.22" } },
						{1830, new List<string> {"weather.clouds 0.7" } },
						{1840, new List<string> {"weather.fog 0.2" } },
						{1850, new List<string> {"weather.clouds 0.68" } },
						{1860, new List<string> {"weather.fog 0.18" } },
						{1870, new List<string> {"weather.clouds 0.66" } },
						{1880, new List<string> {"weather.fog 0.16" } },
						{1890, new List<string> {"weather.clouds 0.64" } },
						{1900, new List<string> {"weather.fog 0.14" } },
						{1910, new List<string> {"weather.clouds 0.62" } },
						{1920, new List<string> {"weather.fog 0.12" } },
						{1930, new List<string> {"weather.clouds 0.6" } },
						{1940, new List<string> {"weather.fog 0.1" } },
						{1950, new List<string> {"weather.clouds 0.58" } },
						{1960, new List<string> {"weather.fog 0.08" } },
						{1970, new List<string> {"weather.clouds 0.56" } },
						{1980, new List<string> {"weather.fog 0.06" } },
						{1990, new List<string> {"weather.clouds 0.54" } },
						{2000, new List<string> {"weather.fog 0.04" } },
						{2010, new List<string> {"weather.clouds 0.52" } },
						{2020, new List<string> {"weather.fog 0.02" } },
						{2030, new List<string> {"weather.clouds 0.5" } },
						{2040, new List<string> {"weather.fog 0" } },
						{2050, new List<string> {"weather.clouds 0.48" } },
						{2060, new List<string> {"weather.clouds 0.46" } },
						{2070, new List<string> {"weather.clouds 0.44" } },
						{2080, new List<string> {"weather.clouds 0.42" } },
						{2090, new List<string> {"weather.clouds 0.4" } },
						{2100, new List<string> {"weather.clouds 0.38" } },
						{2110, new List<string> {"weather.clouds 0.36" } },
						{2120, new List<string> {"weather.clouds 0.34" } },
						{2130, new List<string> {"weather.clouds 0.32" } },
						{2140, new List<string> {"weather.clouds 0.3" } },
						{2150, new List<string> {"weather.clouds 0.28" } },
						{2160, new List<string> {"weather.clouds 0.26" } },
						{2170, new List<string> {"weather.clouds 0.24" } },
						{2180, new List<string> {"weather.clouds 0.22" } },
						{2190, new List<string> {"weather.clouds 0.2" } },
						{2200, new List<string> {"weather.clouds 0.18" } },
						{2210, new List<string> {"weather.clouds 0.16" } },
						{2220, new List<string> {"weather.clouds 0.14" } },
						{2230, new List<string> {"weather.clouds 0.12" } },
						{2240, new List<string> {"weather.clouds 0.1" } },
						{2250, new List<string> {"weather.clouds 0.08" } },
						{2260, new List<string> {"weather.clouds 0.06" } },
						{2270, new List<string> {"weather.clouds 0.04" } },
						{2280, new List<string> {"weather.clouds 0.02" } },						
                        {2290, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
                    {"longstorm", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.02" } },
						{10, new List<string> {"weather.clouds 0.04" } },
						{20, new List<string> {"weather.clouds 0.06" } },
						{30, new List<string> {"weather.clouds 0.08" } },
						{40, new List<string> {"weather.clouds 0.1" } },
						{50, new List<string> {"weather.clouds 0.12" } },
						{60, new List<string> {"weather.clouds 0.14" } },
						{70, new List<string> {"weather.clouds 0.16" } },
						{80, new List<string> {"weather.clouds 0.18" } },
						{90, new List<string> {"weather.clouds 0.2" } },
						{100, new List<string> {"weather.clouds 0.22" } },
						{110, new List<string> {"weather.clouds 0.24" } },
						{120, new List<string> {"weather.clouds 0.26" } },
						{130, new List<string> {"weather.clouds 0.28" } },
						{140, new List<string> {"weather.clouds 0.3" } },
						{150, new List<string> {"weather.clouds 0.32" } },
						{160, new List<string> {"weather.clouds 0.34" } },
						{170, new List<string> {"weather.clouds 0.36" } },
						{180, new List<string> {"weather.clouds 0.38" } },
						{190, new List<string> {"weather.clouds 0.4" } },
						{200, new List<string> {"weather.clouds 0.42" } },
						{210, new List<string> {"weather.clouds 0.44" } },
						{220, new List<string> {"weather.clouds 0.46" } },
						{230, new List<string> {"weather.clouds 0.48" } },
						{240, new List<string> {"weather.clouds 0.5" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.clouds 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.clouds 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.clouds 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.clouds 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.clouds 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.clouds 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.clouds 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.clouds 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.clouds 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.clouds 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.clouds 0.74" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.clouds 0.76" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{520, new List<string> {"weather.clouds 0.78" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{540, new List<string> {"weather.clouds 0.8" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{560, new List<string> {"weather.clouds 0.82" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{580, new List<string> {"weather.clouds 0.84" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{600, new List<string> {"weather.clouds 0.86" } },
						{605, new List<string> {"weather.wind 0.2" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{620, new List<string> {"weather.clouds 0.88" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{640, new List<string> {"weather.clouds 0.9" } },
						{650, new List<string> {"weather.fog 0.42" } },
						{660, new List<string> {"weather.clouds 0.92" } },
						{670, new List<string> {"weather.fog 0.44" } },
						{680, new List<string> {"weather.clouds 0.94" } },
						{690, new List<string> {"weather.fog 0.46" } },
						{700, new List<string> {"weather.clouds 0.96" } },
						{710, new List<string> {"weather.fog 0.48" } },
						{720, new List<string> {"weather.clouds 0.98" } },
						{725, new List<string> {"weather.wind 0.4" } },
						{730, new List<string> {"weather.fog 0.5" } },
						{740, new List<string> {"weather.clouds 1" } },
						{750, new List<string> {"weather.fog 0.52" } },
						{760, new List<string> {"weather.fog 0.54" } },
						{780, new List<string> {"weather.fog 0.56" } },
						{790, new List<string> {"weather.fog 0.58" } },
						{800, new List<string> {"weather.fog 0.6" } },
						{810, new List<string> {"weather.fog 0.62" } },
						{820, new List<string> {"weather.fog 0.64" } },
						{830, new List<string> {"weather.fog 0.66" } },
						{840, new List<string> {"weather.fog 0.68" } },
						{850, new List<string> {"weather.fog 0.7", "weather.wind 0.6" } },
						{860, new List<string> {"weather.fog 0.72" } },
						{870, new List<string> {"weather.fog 0.74" } },
						{880, new List<string> {"weather.fog 0.76" } },
						{890, new List<string> {"weather.fog 0.78" } },
						{900, new List<string> {"weather.fog 0.8" } },
						{905, new List<string> {"weather.wind 0.8" } },
						{910, new List<string> {"weather.fog 0.82" } },
						{920, new List<string> {"weather.fog 0.84" } },
						{930, new List<string> {"weather.fog 0.86" } },
						{940, new List<string> {"weather.fog 0.88" } },
						{950, new List<string> {"weather.fog 0.9" } },
						{960, new List<string> {"weather.fog 0.92" } },
						{965, new List<string> {"weather.wind 1" } },
						{970, new List<string> {"weather.fog 0.94" } },
						{980, new List<string> {"weather.fog 0.96" } },
						{990, new List<string> {"weather.fog 0.98" } },
						{1000, new List<string> {"weather.fog 1" } },
						{1600, new List<string> {"weather.fog 0.98" } },
						{1610, new List<string> {"weather.fog 0.96" } },
						{1620, new List<string> {"weather.fog 0.94" } },
						{1630, new List<string> {"weather.fog 0.92" } },
						{1640, new List<string> {"weather.fog 0.9" } },
						{1650, new List<string> {"weather.fog 0.88" } },
						{1660, new List<string> {"weather.fog 0.86" } },
						{1665, new List<string> {"weather.wind 0.8" } },
						{1670, new List<string> {"weather.fog 0.84" } },
						{1680, new List<string> {"weather.fog 0.82" } },
						{1690, new List<string> {"weather.fog 0.8" } },
						{1695, new List<string> {"weather.wind 0.6" } },
						{1700, new List<string> {"weather.fog 0.78" } },
						{1710, new List<string> {"weather.fog 0.76" } },
						{1720, new List<string> {"weather.fog 0.74" } },
						{1730, new List<string> {"weather.fog 0.72" } },
						{1740, new List<string> {"weather.fog 0.7" } },
						{1750, new List<string> {"weather.fog 0.68" } },
						{1760, new List<string> {"weather.fog 0.66", "weather.wind 0.4" } },
						{1770, new List<string> {"weather.fog 0.64" } },
						{1780, new List<string> {"weather.fog 0.62" } },
						{1790, new List<string> {"weather.fog 0.6" } },
						{1800, new List<string> {"weather.fog 0.58" } },
						{1810, new List<string> {"weather.fog 0.56" } },
						{1820, new List<string> {"weather.fog 0.54" } },
						{1830, new List<string> {"weather.fog 0.52" } },
						{1840, new List<string> {"weather.fog 0.5", "weather.wind 0.2" } },
						{1850, new List<string> {"weather.clouds 0.98" } },
						{1860, new List<string> {"weather.fog 0.48" } },
						{1870, new List<string> {"weather.clouds 0.96" } },
						{1880, new List<string> {"weather.fog 0.46" } },
						{1890, new List<string> {"weather.clouds 0.94" } },
						{1900, new List<string> {"weather.fog 0.44" } },
						{1910, new List<string> {"weather.clouds 0.92" } },
						{1920, new List<string> {"weather.fog 0.42" } },
						{1930, new List<string> {"weather.clouds 0.9" } },
						{1935, new List<string> {"weather.wind 0" } },
						{1940, new List<string> {"weather.fog 0.4" } },
						{1950, new List<string> {"weather.clouds 0.88" } },
						{1960, new List<string> {"weather.fog 0.38" } },
						{1970, new List<string> {"weather.clouds 0.86" } },
						{1980, new List<string> {"weather.fog 0.36" } },
						{1990, new List<string> {"weather.clouds 0.84" } },
						{2000, new List<string> {"weather.fog 0.34" } },
						{2010, new List<string> {"weather.clouds 0.82" } },
						{2020, new List<string> {"weather.fog 0.32" } },
						{2030, new List<string> {"weather.clouds 0.8" } },
						{2040, new List<string> {"weather.fog 0.3" } },
						{2050, new List<string> {"weather.clouds 0.78" } },
						{2060, new List<string> {"weather.fog 0.28" } },
						{2070, new List<string> {"weather.clouds 0.76" } },
						{2080, new List<string> {"weather.fog 0.26" } },
						{2090, new List<string> {"weather.clouds 0.74" } },
						{2100, new List<string> {"weather.fog 0.24" } },
						{2110, new List<string> {"weather.clouds 0.72" } },
						{2120, new List<string> {"weather.fog 0.22" } },
						{2130, new List<string> {"weather.clouds 0.7" } },
						{2140, new List<string> {"weather.fog 0.2" } },
						{2150, new List<string> {"weather.clouds 0.68" } },
						{2160, new List<string> {"weather.fog 0.18" } },
						{2170, new List<string> {"weather.clouds 0.66" } },
						{2180, new List<string> {"weather.fog 0.16" } },
						{2190, new List<string> {"weather.clouds 0.64" } },
						{2200, new List<string> {"weather.fog 0.14" } },
						{2210, new List<string> {"weather.clouds 0.62" } },
						{2220, new List<string> {"weather.fog 0.12" } },
						{2230, new List<string> {"weather.clouds 0.6" } },
						{2240, new List<string> {"weather.fog 0.1" } },
						{2250, new List<string> {"weather.clouds 0.58" } },
						{2260, new List<string> {"weather.fog 0.08" } },
						{2270, new List<string> {"weather.clouds 0.56" } },
						{2280, new List<string> {"weather.fog 0.06" } },
						{2290, new List<string> {"weather.clouds 0.54" } },
						{2300, new List<string> {"weather.fog 0.04" } },
						{2310, new List<string> {"weather.clouds 0.52" } },
						{2320, new List<string> {"weather.fog 0.02" } },
						{2330, new List<string> {"weather.clouds 0.5" } },
						{2340, new List<string> {"weather.fog 0" } },
						{2350, new List<string> {"weather.clouds 0.48" } },
						{2360, new List<string> {"weather.clouds 0.46" } },
						{2370, new List<string> {"weather.clouds 0.44" } },
						{2380, new List<string> {"weather.clouds 0.42" } },
						{2390, new List<string> {"weather.clouds 0.4" } },
						{2400, new List<string> {"weather.clouds 0.38" } },
						{2410, new List<string> {"weather.clouds 0.36" } },
						{2420, new List<string> {"weather.clouds 0.34" } },
						{2430, new List<string> {"weather.clouds 0.32" } },
						{2440, new List<string> {"weather.clouds 0.3" } },
						{2450, new List<string> {"weather.clouds 0.28" } },
						{2460, new List<string> {"weather.clouds 0.26" } },
						{2470, new List<string> {"weather.clouds 0.24" } },
						{2480, new List<string> {"weather.clouds 0.22" } },
						{2490, new List<string> {"weather.clouds 0.2" } },
						{2500, new List<string> {"weather.clouds 0.18" } },
						{2510, new List<string> {"weather.clouds 0.16" } },
						{2520, new List<string> {"weather.clouds 0.14" } },
						{2530, new List<string> {"weather.clouds 0.12" } },
						{2540, new List<string> {"weather.clouds 0.1" } },
						{2550, new List<string> {"weather.clouds 0.08" } },
						{2560, new List<string> {"weather.clouds 0.06" } },
						{2570, new List<string> {"weather.clouds 0.04" } },
						{2580, new List<string> {"weather.clouds 0.02" } },						
                        {2590, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
					{"lightrain", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.01" } },
						{10, new List<string> {"weather.clouds 0.02" } },
						{20, new List<string> {"weather.clouds 0.04" } },
						{30, new List<string> {"weather.clouds 0.06" } },
						{40, new List<string> {"weather.clouds 0.08" } },
						{50, new List<string> {"weather.clouds 0.1" } },
						{60, new List<string> {"weather.clouds 0.12" } },
						{70, new List<string> {"weather.clouds 0.14" } },
						{80, new List<string> {"weather.clouds 0.16" } },
						{90, new List<string> {"weather.clouds 0.18" } },
						{100, new List<string> {"weather.clouds 0.2" } },
						{110, new List<string> {"weather.clouds 0.22" } },
						{120, new List<string> {"weather.clouds 0.24" } },
						{130, new List<string> {"weather.clouds 0.26" } },
						{140, new List<string> {"weather.clouds 0.28" } },
						{150, new List<string> {"weather.clouds 0.3" } },
						{160, new List<string> {"weather.clouds 0.32" } },
						{170, new List<string> {"weather.clouds 0.34" } },
						{180, new List<string> {"weather.clouds 0.36" } },
						{190, new List<string> {"weather.clouds 0.38" } },
						{200, new List<string> {"weather.clouds 0.4" } },
						{210, new List<string> {"weather.clouds 0.42" } },
						{220, new List<string> {"weather.clouds 0.44" } },
						{230, new List<string> {"weather.clouds 0.46" } },
						{240, new List<string> {"weather.clouds 0.48" } },
						{250, new List<string> {"weather.clouds 0.5" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.clouds 0.54" } },
						{280, new List<string> {"weather.clouds 0.56" } },
						{290, new List<string> {"weather.clouds 0.58" } },
						{300, new List<string> {"weather.clouds 0.6" } },
						{310, new List<string> {"weather.clouds 0.62" } },
						{320, new List<string> {"weather.clouds 0.64" } },
						{330, new List<string> {"weather.clouds 0.66" } },
						{340, new List<string> {"weather.clouds 0.68" } },
						{350, new List<string> {"weather.clouds 0.7" } },
						{360, new List<string> {"weather.clouds 0.72" } },
						{370, new List<string> {"weather.clouds 0.74" } },
						{380, new List<string> { "weather.clouds 0.75" } },
						{390, new List<string> { "weather.fog 0.02" } },
						{400, new List<string> { "weather.clouds 0.76" } },
						{410, new List<string> { "weather.fog 0.04" } },
						{420, new List<string> { "weather.clouds 0.78" } },
						{430, new List<string> { "weather.fog 0.06" } },
						{440, new List<string> { "weather.clouds 0.8" } },
						{450, new List<string> { "weather.fog 0.08" } },
						{460, new List<string> { "weather.clouds 0.82" } },
						{470, new List<string> { "weather.fog 0.1" } },
						{490, new List<string> { "weather.clouds 0.84" } },
						{500, new List<string> { "weather.fog 0.12" } },
						{520, new List<string> { "weather.clouds 0.86" } },
						{530, new List<string> { "weather.fog 0.14" } },
						{540, new List<string> { "weather.clouds 0.88" } },
						{550, new List<string> { "weather.fog 0.16" } },
						{570, new List<string> { "weather.clouds 0.9" } },
						{580, new List<string> { "weather.fog 0.18" } },
						{600, new List<string> { "weather.clouds 0.92" } },
						{610, new List<string> { "weather.fog 0.2" } },
						{630, new List<string> { "weather.clouds 0.94" } },
						{640, new List<string> { "weather.fog 0.22" } },
						{650, new List<string> { "weather.clouds 0.96" } },
						{660, new List<string> { "weather.fog 0.24" } },
						{670, new List<string> { "weather.clouds 0.98" } },
						{680, new List<string> { "weather.fog 0.26" } },
						{690, new List<string> { "weather.clouds 1" } },
						{700, new List<string> { "weather.fog 0.28" } },
						{710, new List<string> { "weather.fog 0.3" } },
						{720, new List<string> { "weather.fog 0.32" } },
						{730, new List<string> { "weather.fog 0.34" } },
						{740, new List<string> { "weather.fog 0.35" } },
						{1040, new List<string> { "weather.clouds 0.98" } },
						{1050, new List<string> { "weather.clouds 0.96" } },
						{1060, new List<string> { "weather.clouds 0.94" } },
						{1070, new List<string> { "weather.clouds 0.92" } },
						{1080, new List<string> { "weather.clouds 0.9" } },
						{1100, new List<string> { "weather.clouds 0.88" } },
						{1110, new List<string> { "weather.clouds 0.86" } },
						{1120, new List<string> { "weather.clouds 0.84" } },
						{1130, new List<string> { "weather.clouds 0.82" } },
						{1140, new List<string> { "weather.clouds 0.8" } },
						{1160, new List<string> { "weather.clouds 0.78" } },
						{1170, new List<string> { "weather.clouds 0.76" } },
						{1180, new List<string> { "weather.clouds 0.74" } },
						{1190, new List<string> { "weather.clouds 0.72" } },
						{1200, new List<string> { "weather.clouds 0.7" } },
						{1220, new List<string> { "weather.clouds 0.68" } },
						{1230, new List<string> { "weather.clouds 0.66" } },
						{1240, new List<string> { "weather.clouds 0.64" } },
						{1250, new List<string> { "weather.clouds 0.62" } },
						{1270, new List<string> { "weather.clouds 0.6" } },
						{1280, new List<string> { "weather.clouds 0.58" } },
						{1290, new List<string> { "weather.clouds 0.56" } },
						{1300, new List<string> { "weather.clouds 0.54" } },
						{1310, new List<string> { "weather.clouds 0.52" } },
						{1320, new List<string> { "weather.clouds 0.5" } },
						{1340, new List<string> { "weather.fog 0.34" } },
						{1350, new List<string> { "weather.clouds 0.48" } },
						{1360, new List<string> { "weather.fog 0.32" } },
						{1370, new List<string> { "weather.clouds 0.46" } },
						{1380, new List<string> { "weather.fog 0.3" } },
						{1390, new List<string> { "weather.clouds 0.44" } },
						{1400, new List<string> { "weather.fog 0.28" } },
						{1410, new List<string> { "weather.clouds 0.42" } },
						{1420, new List<string> { "weather.fog 0.26" } },
						{1430, new List<string> { "weather.clouds 0.4" } },
						{1440, new List<string> { "weather.fog 0.24" } },
						{1450, new List<string> { "weather.clouds 0.38" } },
						{1460, new List<string> { "weather.fog 0.22" } },
						{1470, new List<string> { "weather.clouds 0.36" } },
						{1480, new List<string> { "weather.fog 0.2" } },
						{1490, new List<string> { "weather.clouds 0.34" } },
						{1500, new List<string> { "weather.fog 0.18" } },
						{1510, new List<string> { "weather.clouds 0.32" } },
						{1520, new List<string> { "weather.fog 0.16" } },
						{1530, new List<string> { "weather.clouds 0.3" } },
						{1540, new List<string> { "weather.fog 0.14" } },
						{1550, new List<string> { "weather.clouds 0.28" } },
						{1560, new List<string> { "weather.fog 0.12" } },
						{1570, new List<string> { "weather.clouds 0.26" } },
						{1580, new List<string> { "weather.fog 0.1" } },
						{1590, new List<string> { "weather.clouds 0.24" } },
						{1600, new List<string> { "weather.fog 0.08" } },
						{1610, new List<string> { "weather.clouds 0.22" } },
						{1620, new List<string> { "weather.fog 0.06" } },
						{1630, new List<string> { "weather.clouds 0.2" } },
						{1640, new List<string> { "weather.fog 0.04" } },
						{1650, new List<string> { "weather.clouds 0.18" } },
						{1660, new List<string> { "weather.fog 0.02" } },
						{1670, new List<string> { "weather.clouds 0.16" } },
						{1680, new List<string> { "weather.fog 0.01" } },
						{1690, new List<string> { "weather.clouds 0.14" } },
						{1700, new List<string> { "weather.fog 0" } },
						{1710, new List<string> { "weather.clouds 0.12" } },
						{1720, new List<string> { "weather.clouds 0.1" } },
						{1730, new List<string> { "weather.clouds 0.08" } },
						{1740, new List<string> { "weather.clouds 0.06" } },
						{1750, new List<string> { "weather.clouds 0.04" } },
						{1760, new List<string> { "weather.clouds 0.02" } },
                        {1770, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
                    {"rain", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> { "weather.clouds 0.05" } },
                        {30, new List<string> { "weather.clouds 0.1" } },
                        {60, new List<string> { "weather.clouds 0.15" } },
                        {90, new List<string> { "weather.clouds 0.2" } },
                        {120, new List<string> { "weather.clouds 0.25" } },
                        {150, new List<string> { "weather.clouds 0.3" } },
                        {180, new List<string> { "weather.clouds 0.35" } },
                        {210, new List<string> { "weather.clouds 0.4" } },
                        {240, new List<string> { "weather.clouds 0.45" } },
                        {270, new List<string> { "weather.clouds 0.5" } },
                        {300, new List<string> { "weather.clouds 0.55" } },
                        {330, new List<string> { "weather.clouds 0.6" } },
						{360, new List<string> { "weather.clouds 0.85" } },
                        {390, new List<string> { "weather.fog 0.05" } },
                        {410, new List<string> { "weather.clouds 0.9" } },
                        {420, new List<string> { "weather.fog 0.1" } },
                        {450, new List<string> { "weather.fog 0.15" } },
                        {440, new List<string> { "weather.clouds 0.95" } },
                        {470, new List<string> { "weather.clouds 1" } },
                        {480, new List<string> { "weather.fog 0.2" } },
						{500, new List<string> { "weather.fog 0.25" } },
						{520, new List<string> { "weather.fog 0.3" } },
						{540, new List<string> { "weather.fog 0.35" } },
						{560, new List<string> { "weather.fog 0.4" } },
						{580, new List<string> { "weather.fog 0.45" } },
						{600, new List<string> { "weather.fog 0.5" } },
						{980, new List<string> { "weather.clouds 0.95" } },
						{995, new List<string> { "weather.clouds 0.9" } },
						{1000, new List<string> { "weather.fog 0.45" } },
						{1010, new List<string> { "weather.clouds 0.85" } },
						{1030, new List<string> { "weather.fog 0.4" } },
						{1040, new List<string> { "weather.clouds 0.80" } },
						{1060, new List<string> { "weather.fog 0.35" } },
						{1070, new List<string> { "weather.clouds 0.75" } },
						{1090, new List<string> { "weather.fog 0.25" } },
						{1100, new List<string> { "weather.clouds 0.7" } },
						{1110, new List<string> { "weather.fog 0.2" } },
						{1120, new List<string> { "weather.clouds 0.65" } },
						{1130, new List<string> { "weather.fog 0.15" } },
						{1140, new List<string> { "weather.clouds 0.6" } },
						{1150, new List<string> { "weather.fog 0.1" } },
						{1160, new List<string> { "weather.clouds 0.55" } },
						{1170, new List<string> { "weather.fog 0.05" } },
						{1180, new List<string> { "weather.clouds 0.5" } },
						{1200, new List<string> { "weather.fog 0" } },
						{1210, new List<string> { "weather.clouds 0.45" } },
						{1230, new List<string> { "weather.clouds 0.4" } },
						{1250, new List<string> { "weather.clouds 0.35" } },
						{1270, new List<string> { "weather.clouds 0.3" } },
						{1290, new List<string> { "weather.clouds 0.25" } },
						{1310, new List<string> { "weather.clouds 0.20" } },
						{1330, new List<string> { "weather.clouds 0.15" } },
						{1350, new List<string> { "weather.clouds 0.1" } },
						{1370, new List<string> { "weather.clouds 0.05" } },
                        {1390, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
                    {"longrain", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> { "weather.clouds 0.05" } },
                        {30, new List<string> { "weather.clouds 0.1" } },
                        {60, new List<string> { "weather.clouds 0.15" } },
                        {90, new List<string> { "weather.clouds 0.2" } },
                        {120, new List<string> { "weather.clouds 0.25" } },
                        {150, new List<string> { "weather.clouds 0.3" } },
                        {180, new List<string> { "weather.clouds 0.35" } },
                        {210, new List<string> { "weather.clouds 0.4" } },
                        {240, new List<string> { "weather.clouds 0.45" } },
                        {270, new List<string> { "weather.clouds 0.5" } },
                        {300, new List<string> { "weather.clouds 0.55" } },
                        {330, new List<string> { "weather.clouds 0.6" } },
						{360, new List<string> { "weather.clouds 0.85" } },
                        {390, new List<string> { "weather.fog 0.0" } },
                        {410, new List<string> { "weather.clouds 0.9" } },
                        {420, new List<string> { "weather.fog 0.1" } },
                        {450, new List<string> { "weather.fog 0.15" } },
                        {440, new List<string> { "weather.clouds 0.95" } },
                        {470, new List<string> { "weather.clouds 1" } },
                        {480, new List<string> { "weather.fog 0.2" } },
						{500, new List<string> { "weather.fog 0.25" } },
						{520, new List<string> { "weather.fog 0.3" } },
						{540, new List<string> { "weather.fog 0.35" } },
						{560, new List<string> { "weather.fog 0.4" } },
						{580, new List<string> { "weather.fog 0.45" } },
						{600, new List<string> { "weather.fog 0.5" } },
						{1580, new List<string> { "weather.clouds 0.95" } },
						{1595, new List<string> { "weather.clouds 0.9" } },
						{1600, new List<string> { "weather.fog 0.45" } },
						{1610, new List<string> { "weather.clouds 0.85" } },
						{1630, new List<string> { "weather.fog 0.4" } },
						{1640, new List<string> { "weather.clouds 0.80" } },
						{1660, new List<string> { "weather.fog 0.35" } },
						{1670, new List<string> { "weather.clouds 0.75" } },
						{1690, new List<string> { "weather.fog 0.25" } },
						{1700, new List<string> { "weather.clouds 0.7" } },
						{1710, new List<string> { "weather.fog 0.2" } },
						{1720, new List<string> { "weather.clouds 0.65" } },
						{1730, new List<string> { "weather.fog 0.15" } },
						{1740, new List<string> { "weather.clouds 0.6" } },
						{1750, new List<string> { "weather.fog 0.1" } },
						{1760, new List<string> { "weather.clouds 0.55" } },
						{1770, new List<string> { "weather.fog 0.05" } },
						{1780, new List<string> { "weather.clouds 0.5" } },
						{1800, new List<string> { "weather.fog 0" } },
						{1810, new List<string> { "weather.clouds 0.45" } },
						{1830, new List<string> { "weather.clouds 0.4" } },
						{1850, new List<string> { "weather.clouds 0.35" } },
						{1870, new List<string> { "weather.clouds 0.3" } },
						{1890, new List<string> { "weather.clouds 0.25" } },
						{1910, new List<string> { "weather.clouds 0.20" } },
						{1930, new List<string> { "weather.clouds 0.15" } },
						{1950, new List<string> { "weather.clouds 0.1" } },
						{1970, new List<string> { "weather.clouds 0.05" } },
                        {1990, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
					{"lightfog", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.02" } },
						{10, new List<string> {"weather.clouds 0.04" } },
						{20, new List<string> {"weather.clouds 0.06" } },
						{30, new List<string> {"weather.clouds 0.08" } },
						{40, new List<string> {"weather.clouds 0.1" } },
						{50, new List<string> {"weather.clouds 0.12" } },
						{60, new List<string> {"weather.clouds 0.14" } },
						{70, new List<string> {"weather.clouds 0.16" } },
						{80, new List<string> {"weather.clouds 0.18" } },
						{90, new List<string> {"weather.clouds 0.2" } },
						{100, new List<string> {"weather.clouds 0.22" } },
						{110, new List<string> {"weather.clouds 0.24" } },
						{120, new List<string> {"weather.clouds 0.26" } },
						{130, new List<string> {"weather.clouds 0.28" } },
						{140, new List<string> {"weather.clouds 0.3" } },
						{150, new List<string> {"weather.clouds 0.32" } },
						{160, new List<string> {"weather.clouds 0.34" } },
						{170, new List<string> {"weather.clouds 0.36" } },
						{180, new List<string> {"weather.clouds 0.38" } },
						{190, new List<string> {"weather.clouds 0.4" } },
						{200, new List<string> {"weather.clouds 0.42" } },
						{210, new List<string> {"weather.clouds 0.44" } },
						{220, new List<string> {"weather.clouds 0.46" } },
						{230, new List<string> {"weather.clouds 0.48" } },
						{240, new List<string> {"weather.clouds 0.5" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.clouds 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.clouds 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.clouds 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.clouds 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.clouds 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.clouds 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.clouds 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.clouds 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.clouds 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.clouds 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.clouds 0.74" } },
						{490, new List<string> {"weather.fog 0.25" } },
						{500, new List<string> {"weather.clouds 0.75" } },
						{1190, new List<string> {"weather.clouds 0.74" } },
						{1200, new List<string> {"weather.fog 0.24" } },
						{1210, new List<string> {"weather.clouds 0.72" } },
						{1220, new List<string> {"weather.fog 0.22" } },
						{1230, new List<string> {"weather.clouds 0.7" } },
						{1240, new List<string> {"weather.fog 0.2" } },
						{1250, new List<string> {"weather.clouds 0.68" } },
						{1260, new List<string> {"weather.fog 0.18" } },
						{1270, new List<string> {"weather.clouds 0.66" } },
						{1280, new List<string> {"weather.fog 0.16" } },
						{1290, new List<string> {"weather.clouds 0.64" } },
						{1300, new List<string> {"weather.fog 0.14" } },
						{1310, new List<string> {"weather.clouds 0.62" } },
						{1320, new List<string> {"weather.fog 0.12" } },
						{1330, new List<string> {"weather.clouds 0.6" } },
						{1340, new List<string> {"weather.fog 0.1" } },
						{1350, new List<string> {"weather.clouds 0.58" } },
						{1360, new List<string> {"weather.fog 0.08" } },
						{1370, new List<string> {"weather.clouds 0.56" } },
						{1380, new List<string> {"weather.fog 0.06" } },
						{1390, new List<string> {"weather.clouds 0.54" } },
						{1400, new List<string> {"weather.fog 0.04" } },
						{1410, new List<string> {"weather.clouds 0.52" } },
						{1420, new List<string> {"weather.fog 0.02" } },
						{1430, new List<string> {"weather.clouds 0.5" } },
						{1440, new List<string> {"weather.fog 0" } },
						{1450, new List<string> {"weather.clouds 0.48" } },
						{1460, new List<string> {"weather.clouds 0.46" } },
						{1470, new List<string> {"weather.clouds 0.44" } },
						{1480, new List<string> {"weather.clouds 0.42" } },
						{1490, new List<string> {"weather.clouds 0.4" } },
						{1500, new List<string> {"weather.clouds 0.38" } },
						{1510, new List<string> {"weather.clouds 0.36" } },
						{1520, new List<string> {"weather.clouds 0.34" } },
						{1530, new List<string> {"weather.clouds 0.32" } },
						{1540, new List<string> {"weather.clouds 0.3" } },
						{1550, new List<string> {"weather.clouds 0.28" } },
						{1560, new List<string> {"weather.clouds 0.26" } },
						{1570, new List<string> {"weather.clouds 0.24" } },
						{1580, new List<string> {"weather.clouds 0.22" } },
						{1590, new List<string> {"weather.clouds 0.2" } },
						{1600, new List<string> {"weather.clouds 0.18" } },
						{1610, new List<string> {"weather.clouds 0.16" } },
						{1620, new List<string> {"weather.clouds 0.14" } },
						{1630, new List<string> {"weather.clouds 0.12" } },
						{1640, new List<string> {"weather.clouds 0.1" } },
						{1650, new List<string> {"weather.clouds 0.08" } },
						{1660, new List<string> {"weather.clouds 0.06" } },
						{1670, new List<string> {"weather.clouds 0.04" } },
						{1680, new List<string> {"weather.clouds 0.02" } },						
                        {1690, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
					{"fog", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.02" } },
						{10, new List<string> {"weather.clouds 0.04" } },
						{20, new List<string> {"weather.clouds 0.06" } },
						{30, new List<string> {"weather.clouds 0.08" } },
						{40, new List<string> {"weather.clouds 0.1" } },
						{50, new List<string> {"weather.clouds 0.12" } },
						{60, new List<string> {"weather.clouds 0.14" } },
						{70, new List<string> {"weather.clouds 0.16" } },
						{80, new List<string> {"weather.clouds 0.18" } },
						{90, new List<string> {"weather.clouds 0.2" } },
						{100, new List<string> {"weather.clouds 0.22" } },
						{110, new List<string> {"weather.clouds 0.24" } },
						{120, new List<string> {"weather.clouds 0.26" } },
						{130, new List<string> {"weather.clouds 0.28" } },
						{140, new List<string> {"weather.clouds 0.3" } },
						{150, new List<string> {"weather.clouds 0.32" } },
						{160, new List<string> {"weather.clouds 0.34" } },
						{170, new List<string> {"weather.clouds 0.36" } },
						{180, new List<string> {"weather.clouds 0.38" } },
						{190, new List<string> {"weather.clouds 0.4" } },
						{200, new List<string> {"weather.clouds 0.42" } },
						{210, new List<string> {"weather.clouds 0.44" } },
						{220, new List<string> {"weather.clouds 0.46" } },
						{230, new List<string> {"weather.clouds 0.48" } },
						{240, new List<string> {"weather.clouds 0.5" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.clouds 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.clouds 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.clouds 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.clouds 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.clouds 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.clouds 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.clouds 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.clouds 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.clouds 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.clouds 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.clouds 0.74" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.clouds 0.76" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{520, new List<string> {"weather.clouds 0.78" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{540, new List<string> {"weather.clouds 0.8" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{560, new List<string> {"weather.clouds 0.82" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{580, new List<string> {"weather.clouds 0.84" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{600, new List<string> {"weather.clouds 0.86" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{620, new List<string> {"weather.clouds 0.88" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{640, new List<string> {"weather.clouds 0.9" } },
						{650, new List<string> {"weather.fog 0.42" } },
						{660, new List<string> {"weather.clouds 0.92" } },
						{670, new List<string> {"weather.fog 0.44" } },
						{680, new List<string> {"weather.clouds 0.94" } },
						{690, new List<string> {"weather.fog 0.46" } },
						{700, new List<string> {"weather.clouds 0.96" } },
						{710, new List<string> {"weather.fog 0.48" } },
						{720, new List<string> {"weather.clouds 0.98" } },
						{730, new List<string> {"weather.fog 0.5" } },
						{740, new List<string> {"weather.clouds 1" } },
						{1350, new List<string> {"weather.clouds 0.98" } },
						{1360, new List<string> {"weather.fog 0.48" } },
						{1370, new List<string> {"weather.clouds 0.96" } },
						{1380, new List<string> {"weather.fog 0.46" } },
						{1390, new List<string> {"weather.clouds 0.94" } },
						{1400, new List<string> {"weather.fog 0.44" } },
						{1410, new List<string> {"weather.clouds 0.92" } },
						{1420, new List<string> {"weather.fog 0.42" } },
						{1430, new List<string> {"weather.clouds 0.9" } },
						{1440, new List<string> {"weather.fog 0.4" } },
						{1450, new List<string> {"weather.clouds 0.88" } },
						{1460, new List<string> {"weather.fog 0.38" } },
						{1470, new List<string> {"weather.clouds 0.86" } },
						{1480, new List<string> {"weather.fog 0.36" } },
						{1490, new List<string> {"weather.clouds 0.84" } },
						{1500, new List<string> {"weather.fog 0.34" } },
						{1510, new List<string> {"weather.clouds 0.82" } },
						{1520, new List<string> {"weather.fog 0.32" } },
						{1530, new List<string> {"weather.clouds 0.8" } },
						{1540, new List<string> {"weather.fog 0.3" } },
						{1550, new List<string> {"weather.clouds 0.78" } },
						{1560, new List<string> {"weather.fog 0.28" } },
						{1570, new List<string> {"weather.clouds 0.76" } },
						{1580, new List<string> {"weather.fog 0.26" } },
						{1590, new List<string> {"weather.clouds 0.74" } },
						{1600, new List<string> {"weather.fog 0.24" } },
						{1610, new List<string> {"weather.clouds 0.72" } },
						{1620, new List<string> {"weather.fog 0.22" } },
						{1630, new List<string> {"weather.clouds 0.7" } },
						{1640, new List<string> {"weather.fog 0.2" } },
						{1650, new List<string> {"weather.clouds 0.68" } },
						{1660, new List<string> {"weather.fog 0.18" } },
						{1670, new List<string> {"weather.clouds 0.66" } },
						{1680, new List<string> {"weather.fog 0.16" } },
						{1690, new List<string> {"weather.clouds 0.64" } },
						{1700, new List<string> {"weather.fog 0.14" } },
						{1710, new List<string> {"weather.clouds 0.62" } },
						{1720, new List<string> {"weather.fog 0.12" } },
						{1730, new List<string> {"weather.clouds 0.6" } },
						{1740, new List<string> {"weather.fog 0.1" } },
						{1750, new List<string> {"weather.clouds 0.58" } },
						{1760, new List<string> {"weather.fog 0.08" } },
						{1770, new List<string> {"weather.clouds 0.56" } },
						{1780, new List<string> {"weather.fog 0.06" } },
						{1790, new List<string> {"weather.clouds 0.54" } },
						{1800, new List<string> {"weather.fog 0.04" } },
						{1810, new List<string> {"weather.clouds 0.52" } },
						{1820, new List<string> {"weather.fog 0.02" } },
						{1830, new List<string> {"weather.clouds 0.5" } },
						{1840, new List<string> {"weather.fog 0" } },
						{1850, new List<string> {"weather.clouds 0.48" } },
						{1860, new List<string> {"weather.clouds 0.46" } },
						{1870, new List<string> {"weather.clouds 0.44" } },
						{1880, new List<string> {"weather.clouds 0.42" } },
						{1890, new List<string> {"weather.clouds 0.4" } },
						{1900, new List<string> {"weather.clouds 0.38" } },
						{1910, new List<string> {"weather.clouds 0.36" } },
						{1920, new List<string> {"weather.clouds 0.34" } },
						{1930, new List<string> {"weather.clouds 0.32" } },
						{1940, new List<string> {"weather.clouds 0.3" } },
						{1950, new List<string> {"weather.clouds 0.28" } },
						{1960, new List<string> {"weather.clouds 0.26" } },
						{1970, new List<string> {"weather.clouds 0.24" } },
						{1980, new List<string> {"weather.clouds 0.22" } },
						{1990, new List<string> {"weather.clouds 0.2" } },
						{2000, new List<string> {"weather.clouds 0.18" } },
						{2010, new List<string> {"weather.clouds 0.16" } },
						{2020, new List<string> {"weather.clouds 0.14" } },
						{2030, new List<string> {"weather.clouds 0.12" } },
						{2040, new List<string> {"weather.clouds 0.1" } },
						{2050, new List<string> {"weather.clouds 0.08" } },
						{2060, new List<string> {"weather.clouds 0.06" } },
						{2070, new List<string> {"weather.clouds 0.04" } },
						{2080, new List<string> {"weather.clouds 0.02" } },						
                        {2090, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
					{"heavyfog", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.02" } },
						{10, new List<string> {"weather.clouds 0.04" } },
						{20, new List<string> {"weather.clouds 0.06" } },
						{30, new List<string> {"weather.clouds 0.08" } },
						{40, new List<string> {"weather.clouds 0.1" } },
						{50, new List<string> {"weather.clouds 0.12" } },
						{60, new List<string> {"weather.clouds 0.14" } },
						{70, new List<string> {"weather.clouds 0.16" } },
						{80, new List<string> {"weather.clouds 0.18" } },
						{90, new List<string> {"weather.clouds 0.2" } },
						{100, new List<string> {"weather.clouds 0.22" } },
						{110, new List<string> {"weather.clouds 0.24" } },
						{120, new List<string> {"weather.clouds 0.26" } },
						{130, new List<string> {"weather.clouds 0.28" } },
						{140, new List<string> {"weather.clouds 0.3" } },
						{150, new List<string> {"weather.clouds 0.32" } },
						{160, new List<string> {"weather.clouds 0.34" } },
						{170, new List<string> {"weather.clouds 0.36" } },
						{180, new List<string> {"weather.clouds 0.38" } },
						{190, new List<string> {"weather.clouds 0.4" } },
						{200, new List<string> {"weather.clouds 0.42" } },
						{210, new List<string> {"weather.clouds 0.44" } },
						{220, new List<string> {"weather.clouds 0.46" } },
						{230, new List<string> {"weather.clouds 0.48" } },
						{240, new List<string> {"weather.clouds 0.5" } },
						{250, new List<string> {"weather.fog 0.02" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.fog 0.04" } },
						{280, new List<string> {"weather.clouds 0.54" } },
						{290, new List<string> {"weather.fog 0.06" } },
						{300, new List<string> {"weather.clouds 0.56" } },
						{310, new List<string> {"weather.fog 0.08" } },
						{320, new List<string> {"weather.clouds 0.58" } },
						{330, new List<string> {"weather.fog 0.1" } },
						{340, new List<string> {"weather.clouds 0.6" } },
						{350, new List<string> {"weather.fog 0.12" } },
						{360, new List<string> {"weather.clouds 0.62" } },
						{370, new List<string> {"weather.fog 0.14" } },
						{380, new List<string> {"weather.clouds 0.64" } },
						{390, new List<string> {"weather.fog 0.16" } },
						{400, new List<string> {"weather.clouds 0.66" } },
						{410, new List<string> {"weather.fog 0.18" } },
						{420, new List<string> {"weather.clouds 0.68" } },
						{430, new List<string> {"weather.fog 0.20" } },
						{440, new List<string> {"weather.clouds 0.7" } },
						{450, new List<string> {"weather.fog 0.22" } },
						{460, new List<string> {"weather.clouds 0.72" } },
						{470, new List<string> {"weather.fog 0.24" } },
						{480, new List<string> {"weather.clouds 0.74" } },
						{490, new List<string> {"weather.fog 0.26" } },
						{500, new List<string> {"weather.clouds 0.76" } },
						{510, new List<string> {"weather.fog 0.28" } },
						{520, new List<string> {"weather.clouds 0.78" } },
						{530, new List<string> {"weather.fog 0.3" } },
						{540, new List<string> {"weather.clouds 0.8" } },
						{550, new List<string> {"weather.fog 0.32" } },
						{560, new List<string> {"weather.clouds 0.82" } },
						{570, new List<string> {"weather.fog 0.34" } },
						{580, new List<string> {"weather.clouds 0.84" } },
						{590, new List<string> {"weather.fog 0.36" } },
						{600, new List<string> {"weather.clouds 0.86" } },
						{610, new List<string> {"weather.fog 0.38" } },
						{620, new List<string> {"weather.clouds 0.88" } },
						{630, new List<string> {"weather.fog 0.4" } },
						{640, new List<string> {"weather.clouds 0.9" } },
						{650, new List<string> {"weather.fog 0.42" } },
						{660, new List<string> {"weather.clouds 0.92" } },
						{670, new List<string> {"weather.fog 0.44" } },
						{680, new List<string> {"weather.clouds 0.94" } },
						{690, new List<string> {"weather.fog 0.46" } },
						{700, new List<string> {"weather.clouds 0.96" } },
						{710, new List<string> {"weather.fog 0.48" } },
						{720, new List<string> {"weather.clouds 0.98" } },
						{730, new List<string> {"weather.fog 0.5" } },
						{740, new List<string> {"weather.clouds 1" } },
						{750, new List<string> {"weather.fog 0.52" } },
						{760, new List<string> {"weather.fog 0.54" } },
						{780, new List<string> {"weather.fog 0.56" } },
						{790, new List<string> {"weather.fog 0.58" } },
						{800, new List<string> {"weather.fog 0.6" } },
						{810, new List<string> {"weather.fog 0.62" } },
						{820, new List<string> {"weather.fog 0.64" } },
						{830, new List<string> {"weather.fog 0.66" } },
						{840, new List<string> {"weather.fog 0.68" } },
						{850, new List<string> {"weather.fog 0.7" } },
						{860, new List<string> {"weather.fog 0.72" } },
						{870, new List<string> {"weather.fog 0.74" } },
						{880, new List<string> {"weather.fog 0.76" } },
						{890, new List<string> {"weather.fog 0.78" } },
						{900, new List<string> {"weather.fog 0.8" } },
						{910, new List<string> {"weather.fog 0.82" } },
						{920, new List<string> {"weather.fog 0.84" } },
						{930, new List<string> {"weather.fog 0.86" } },
						{940, new List<string> {"weather.fog 0.88" } },
						{950, new List<string> {"weather.fog 0.9" } },
						{960, new List<string> {"weather.fog 0.92" } },
						{970, new List<string> {"weather.fog 0.94" } },
						{980, new List<string> {"weather.fog 0.96" } },
						{990, new List<string> {"weather.fog 0.98" } },
						{1000, new List<string> {"weather.fog 1" } },
						{1300, new List<string> {"weather.fog 0.98" } },
						{1310, new List<string> {"weather.fog 0.96" } },
						{1320, new List<string> {"weather.fog 0.94" } },
						{1330, new List<string> {"weather.fog 0.92" } },
						{1340, new List<string> {"weather.fog 0.9" } },
						{1350, new List<string> {"weather.fog 0.88" } },
						{1360, new List<string> {"weather.fog 0.86" } },
						{1370, new List<string> {"weather.fog 0.84" } },
						{1380, new List<string> {"weather.fog 0.82" } },
						{1390, new List<string> {"weather.fog 0.8" } },
						{1400, new List<string> {"weather.fog 0.78" } },
						{1410, new List<string> {"weather.fog 0.76" } },
						{1420, new List<string> {"weather.fog 0.74" } },
						{1430, new List<string> {"weather.fog 0.72" } },
						{1440, new List<string> {"weather.fog 0.7" } },
						{1450, new List<string> {"weather.fog 0.68" } },
						{1460, new List<string> {"weather.fog 0.66" } },
						{1470, new List<string> {"weather.fog 0.64" } },
						{1480, new List<string> {"weather.fog 0.62" } },
						{1490, new List<string> {"weather.fog 0.6" } },
						{1500, new List<string> {"weather.fog 0.58" } },
						{1510, new List<string> {"weather.fog 0.56" } },
						{1520, new List<string> {"weather.fog 0.54" } },
						{1530, new List<string> {"weather.fog 0.52" } },
						{1540, new List<string> {"weather.fog 0.5" } },
						{1550, new List<string> {"weather.clouds 0.98" } },
						{1560, new List<string> {"weather.fog 0.48" } },
						{1570, new List<string> {"weather.clouds 0.96" } },
						{1580, new List<string> {"weather.fog 0.46" } },
						{1590, new List<string> {"weather.clouds 0.94" } },
						{1600, new List<string> {"weather.fog 0.44" } },
						{1610, new List<string> {"weather.clouds 0.92" } },
						{1620, new List<string> {"weather.fog 0.42" } },
						{1630, new List<string> {"weather.clouds 0.9" } },
						{1640, new List<string> {"weather.fog 0.4" } },
						{1650, new List<string> {"weather.clouds 0.88" } },
						{1660, new List<string> {"weather.fog 0.38" } },
						{1670, new List<string> {"weather.clouds 0.86" } },
						{1680, new List<string> {"weather.fog 0.36" } },
						{1690, new List<string> {"weather.clouds 0.84" } },
						{1700, new List<string> {"weather.fog 0.34" } },
						{1710, new List<string> {"weather.clouds 0.82" } },
						{1720, new List<string> {"weather.fog 0.32" } },
						{1730, new List<string> {"weather.clouds 0.8" } },
						{1740, new List<string> {"weather.fog 0.3" } },
						{1750, new List<string> {"weather.clouds 0.78" } },
						{1760, new List<string> {"weather.fog 0.28" } },
						{1770, new List<string> {"weather.clouds 0.76" } },
						{1780, new List<string> {"weather.fog 0.26" } },
						{1790, new List<string> {"weather.clouds 0.74" } },
						{1800, new List<string> {"weather.fog 0.24" } },
						{1810, new List<string> {"weather.clouds 0.72" } },
						{1820, new List<string> {"weather.fog 0.22" } },
						{1830, new List<string> {"weather.clouds 0.7" } },
						{1840, new List<string> {"weather.fog 0.2" } },
						{1850, new List<string> {"weather.clouds 0.68" } },
						{1860, new List<string> {"weather.fog 0.18" } },
						{1870, new List<string> {"weather.clouds 0.66" } },
						{1880, new List<string> {"weather.fog 0.16" } },
						{1890, new List<string> {"weather.clouds 0.64" } },
						{1900, new List<string> {"weather.fog 0.14" } },
						{1910, new List<string> {"weather.clouds 0.62" } },
						{1920, new List<string> {"weather.fog 0.12" } },
						{1930, new List<string> {"weather.clouds 0.6" } },
						{1940, new List<string> {"weather.fog 0.1" } },
						{1950, new List<string> {"weather.clouds 0.58" } },
						{1960, new List<string> {"weather.fog 0.08" } },
						{1970, new List<string> {"weather.clouds 0.56" } },
						{1980, new List<string> {"weather.fog 0.06" } },
						{1990, new List<string> {"weather.clouds 0.54" } },
						{2000, new List<string> {"weather.fog 0.04" } },
						{2010, new List<string> {"weather.clouds 0.52" } },
						{2020, new List<string> {"weather.fog 0.02" } },
						{2030, new List<string> {"weather.clouds 0.5" } },
						{2040, new List<string> {"weather.fog 0" } },
						{2050, new List<string> {"weather.clouds 0.48" } },
						{2060, new List<string> {"weather.clouds 0.46" } },
						{2070, new List<string> {"weather.clouds 0.44" } },
						{2080, new List<string> {"weather.clouds 0.42" } },
						{2090, new List<string> {"weather.clouds 0.4" } },
						{2100, new List<string> {"weather.clouds 0.38" } },
						{2110, new List<string> {"weather.clouds 0.36" } },
						{2120, new List<string> {"weather.clouds 0.34" } },
						{2130, new List<string> {"weather.clouds 0.32" } },
						{2140, new List<string> {"weather.clouds 0.3" } },
						{2150, new List<string> {"weather.clouds 0.28" } },
						{2160, new List<string> {"weather.clouds 0.26" } },
						{2170, new List<string> {"weather.clouds 0.24" } },
						{2180, new List<string> {"weather.clouds 0.22" } },
						{2190, new List<string> {"weather.clouds 0.2" } },
						{2200, new List<string> {"weather.clouds 0.18" } },
						{2210, new List<string> {"weather.clouds 0.16" } },
						{2220, new List<string> {"weather.clouds 0.14" } },
						{2230, new List<string> {"weather.clouds 0.12" } },
						{2240, new List<string> {"weather.clouds 0.1" } },
						{2250, new List<string> {"weather.clouds 0.08" } },
						{2260, new List<string> {"weather.clouds 0.06" } },
						{2270, new List<string> {"weather.clouds 0.04" } },
						{2280, new List<string> {"weather.clouds 0.02" } },						
                        {2290, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    },
					{"overcast", new Dictionary<int, List<string>>
                    {
                        {1, new List<string> {"weather.clouds 0.01" } },
						{10, new List<string> {"weather.clouds 0.02" } },
						{20, new List<string> {"weather.clouds 0.04" } },
						{30, new List<string> {"weather.clouds 0.06" } },
						{40, new List<string> {"weather.clouds 0.08" } },
						{50, new List<string> {"weather.clouds 0.1" } },
						{60, new List<string> {"weather.clouds 0.12" } },
						{70, new List<string> {"weather.clouds 0.14" } },
						{80, new List<string> {"weather.clouds 0.16" } },
						{90, new List<string> {"weather.clouds 0.18" } },
						{100, new List<string> {"weather.clouds 0.2" } },
						{110, new List<string> {"weather.clouds 0.22" } },
						{120, new List<string> {"weather.clouds 0.24" } },
						{130, new List<string> {"weather.clouds 0.26" } },
						{140, new List<string> {"weather.clouds 0.28" } },
						{150, new List<string> {"weather.clouds 0.3" } },
						{160, new List<string> {"weather.clouds 0.32" } },
						{170, new List<string> {"weather.clouds 0.34" } },
						{180, new List<string> {"weather.clouds 0.36" } },
						{190, new List<string> {"weather.clouds 0.38" } },
						{200, new List<string> {"weather.clouds 0.4" } },
						{210, new List<string> {"weather.clouds 0.42" } },
						{220, new List<string> {"weather.clouds 0.44" } },
						{230, new List<string> {"weather.clouds 0.46" } },
						{240, new List<string> {"weather.clouds 0.48" } },
						{250, new List<string> {"weather.clouds 0.5" } },
						{260, new List<string> {"weather.clouds 0.52" } },
						{270, new List<string> {"weather.clouds 0.54" } },
						{280, new List<string> {"weather.clouds 0.56" } },
						{290, new List<string> {"weather.clouds 0.58" } },
						{300, new List<string> {"weather.clouds 0.6" } },
						{310, new List<string> {"weather.clouds 0.62" } },
						{320, new List<string> {"weather.clouds 0.64" } },
						{330, new List<string> {"weather.clouds 0.66" } },
						{340, new List<string> {"weather.clouds 0.68" } },
						{350, new List<string> {"weather.clouds 0.7" } },
						{360, new List<string> {"weather.clouds 0.72" } },
						{370, new List<string> {"weather.clouds 0.74" } },
						{380, new List<string> {"weather.clouds 0.76" } },
						{390, new List<string> {"weather.clouds 0.78" } },
						{400, new List<string> {"weather.clouds 0.8" } },
						{410, new List<string> {"weather.clouds 0.82" } },
						{420, new List<string> {"weather.clouds 0.84" } },
						{430, new List<string> {"weather.clouds 0.86" } },
						{440, new List<string> {"weather.clouds 0.88" } },
						{450, new List<string> {"weather.clouds 0.9" } },
						{460, new List<string> {"weather.clouds 0.92" } },
						{470, new List<string> {"weather.clouds 0.94" } },
						{480, new List<string> {"weather.clouds 0.96" } },
						{490, new List<string> {"weather.clouds 0.98" } },
						{500, new List<string> {"weather.clouds 1" } },
						{1100, new List<string> {"weather.clouds 0.98" } },
						{1110, new List<string> {"weather.clouds 0.96" } },
						{1120, new List<string> {"weather.clouds 0.94" } },
						{1130, new List<string> {"weather.clouds 0.92" } },
						{1140, new List<string> {"weather.clouds 0.9" } },
						{1150, new List<string> {"weather.clouds 0.88" } },
						{1160, new List<string> {"weather.clouds 0.86" } },
						{1170, new List<string> {"weather.clouds 0.84" } },
						{1180, new List<string> {"weather.clouds 0.82" } },
						{1190, new List<string> {"weather.clouds 0.8" } },
						{1200, new List<string> {"weather.clouds 0.78" } },
						{1210, new List<string> {"weather.clouds 0.76" } },
						{1220, new List<string> {"weather.clouds 0.74" } },
						{1230, new List<string> {"weather.clouds 0.72" } },
						{1240, new List<string> {"weather.clouds 0.7" } },
						{1250, new List<string> {"weather.clouds 0.68" } },
						{1260, new List<string> {"weather.clouds 0.66" } },
						{1270, new List<string> {"weather.clouds 0.64" } },
						{1280, new List<string> {"weather.clouds 0.62" } },
						{1290, new List<string> {"weather.clouds 0.6" } },
						{1300, new List<string> {"weather.clouds 0.58" } },
						{1310, new List<string> {"weather.clouds 0.56" } },
						{1320, new List<string> {"weather.clouds 0.54" } },
						{1330, new List<string> {"weather.clouds 0.52" } },
						{1340, new List<string> {"weather.clouds 0.5" } },
						{1350, new List<string> {"weather.clouds 0.48" } },
						{1360, new List<string> {"weather.clouds 0.46" } },
						{1370, new List<string> {"weather.clouds 0.44" } },
						{1380, new List<string> {"weather.clouds 0.42" } },
						{1390, new List<string> {"weather.clouds 0.4" } },
						{1400, new List<string> {"weather.clouds 0.38" } },
						{1410, new List<string> {"weather.clouds 0.36" } },
						{1420, new List<string> {"weather.clouds 0.34" } },
						{1430, new List<string> {"weather.clouds 0.32" } },
						{1440, new List<string> {"weather.clouds 0.3" } },
						{1450, new List<string> {"weather.clouds 0.28" } },
						{1460, new List<string> {"weather.clouds 0.26" } },
						{1470, new List<string> {"weather.clouds 0.24" } },
						{1480, new List<string> {"weather.clouds 0.22" } },
						{1490, new List<string> {"weather.clouds 0.2" } },
						{1500, new List<string> {"weather.clouds 0.18" } },
						{1510, new List<string> {"weather.clouds 0.16" } },
						{1520, new List<string> {"weather.clouds 0.14" } },
						{1530, new List<string> {"weather.clouds 0.12" } },
						{1540, new List<string> {"weather.clouds 0.1" } },
						{1550, new List<string> {"weather.clouds 0.08" } },
						{1560, new List<string> {"weather.clouds 0.06" } },
						{1570, new List<string> {"weather.clouds 0.04" } },
						{1580, new List<string> {"weather.clouds 0.02" } },
                        {1590, new List<string> {"weather.clouds 0", "STOP" } },
                    }
                    }
            };
    }
}